@@ -21,65 +21,6 @@
 #include "funcs.h"
 #include "laser.h"
 
-sprite * shield;
-SDL_Surface * images[NUM_IMAGES];
-Mix_Chunk * sounds[NUM_SOUNDS];
-Mix_Music * musics[NUM_MUSICS];
-SDL_Surface * bkgd;
-
-/* --- unload all media --- */
-void laser_unload_data(void) {
-	int i;
-
-	for (i = 0; i < NUM_IMAGES; i++)
-		SDL_FreeSurface(images[i]);
-
-	if (sys_sound) {
-		for (i = 0; i < NUM_SOUNDS; i++)
-			Mix_FreeChunk(sounds[i]);
-		for (i = 0; i < NUM_MUSICS; i++)
-			Mix_FreeMusic(musics[i]);
-	}
-
-	FreeSprite(shield);
-
-	pause_unload_media();
-
-	for ( i=1; i<255; i++ )
-		SDL_FreeSurface( letters[i] );
-
-	TTF_CloseFont(font);
-}
-
-/* --- Load all media --- */
-void laser_load_data(void) {
-	int i;
-
-	font = LoadFont( ttf_font, 32);
-
-	/* Load images: */
-	for (i = 0; i < NUM_IMAGES; i++) 
-		images[i] = LoadImage(image_filenames[i], IMG_ALPHA);
-
-	shield = LoadSprite( "cities/shield", IMG_ALPHA );
-
-	if (sys_sound) {
-		for (i = 0; i < NUM_SOUNDS; i++)
-			sounds[i] = LoadSound(sound_filenames[i]);
-
-		for (i = 0; i < NUM_MUSICS; i++)
-			musics[i] = LoadMusic(music_filenames[i]);
-	}
-
-	pause_load_media();
-
-	for (i=1; i<255; i++) {
-			unsigned char t[2]=" ";
-			t[0]=i;
-			letters[i] = black_outline( t, font, &white );
-		}
-}
-
 
 #define FPS (1000 / 15)   /* 15 fps max */
 #define CITY_EXPL_START 3 * 5  /* Must be mult. of 5 (number of expl frames) */
@@ -89,31 +30,41 @@ void laser_load_data(void) {
 #define LEVEL_START_WAIT_START 20
 #define LASER_START 5
 #define NUM_ANS 8
+#define COMET_ZAP_FONT_SIZE 32
 
-/* Local (to game.c) 'globals': */
+/* Local (to laser.c) 'globals': */
+static sprite* shield = NULL;
+static SDL_Surface* images[NUM_IMAGES] = {NULL};
+static Mix_Chunk* sounds[NUM_SOUNDS] = {NULL};
+static Mix_Music* musics[NUM_MUSICS] = {NULL};
+static SDL_Surface* bkgd = NULL;
+static TTF_Font* font = NULL;
 
-int wave, speed, score, pre_wave_score, num_attackers, distanceMoved;
-unsigned char ans[NUM_ANS];
-int ans_num;
+static int wave, speed, score, pre_wave_score, num_attackers, distanceMoved;
+static wchar_t ans[NUM_ANS];
+static int ans_num;
 
-comet_type comets[MAX_COMETS];
-city_type cities[NUM_CITIES];
-laser_type laser;
+static comet_type comets[MAX_COMETS];
+static city_type cities[NUM_CITIES];
+static laser_type laser;
 
 /* Local function prototypes: */
-
-void laser_reset_level(int DIF_LEVEL);
-void laser_add_comet(int DIF_LEVEL);
-void laser_draw_numbers(unsigned char * str, int x);
-void laser_draw_line(int x1, int y1, int x2, int y2, int r, int g, int b);
-void laser_putpixel(SDL_Surface * surface, int x, int y, Uint32 pixel);
-void laser_draw_console_image(int i);
-void laser_draw_let(unsigned char c, int x, int y);
-void laser_add_score(int inc);
+static void laser_add_comet(int diff_level);
+static void laser_add_score(int inc);
+static void laser_draw_console_image(int i);
+static void laser_draw_let(wchar_t c, int x, int y);
+static void laser_draw_line(int x1, int y1, int x2, int y2, int r, int g, int b);
+static void laser_draw_numbers(const unsigned char* str, int x);
+static void laser_load_data(void);
+static void laser_reset_level(int diff_level);
+static void laser_putpixel(SDL_Surface* surface, int x, int y, Uint32 pixel);
+static void laser_unload_data(void);
 
 /* --- MAIN GAME FUNCTION!!! --- */
 
-int laser_game(int DIF_LEVEL)
+/* TODO modify game to allow longer words (12 chars or so) */
+
+int PlayLaserGame(int diff_level)
 {
 	int i, img, done, quit, frame, lowest, lowest_y, 
 	    tux_img, old_tux_img, tux_pressing, tux_anim, tux_anim_frame,
@@ -121,14 +72,17 @@ int laser_game(int DIF_LEVEL)
 	    num_comets_alive, paused, picked_comet, 
 	    gameover;
 
+	Uint16 key_unicode;
+
 	SDL_Event event;
 	Uint32    last_time, now_time;
 	SDLKey    key;
 	SDL_Rect  src, dest;
-	unsigned char      str[64];
+	/* str[] is a buffer to draw the scores, waves, etc. (don't need wchar_t) */
+	unsigned char str[64]; 
 
 	LOG( "starting Comet Zap game\n" );
-	DOUT( DIF_LEVEL );
+	DOUT( diff_level );
 
 	SDL_ShowCursor(0);
 	laser_load_data();
@@ -184,7 +138,7 @@ int laser_game(int DIF_LEVEL)
 	/* Reset remaining stuff: */
  
 	bkgd = NULL;
-	laser_reset_level(DIF_LEVEL);
+	laser_reset_level(diff_level);
   
 	/* --- MAIN GAME LOOP!!! --- */
   
@@ -197,7 +151,7 @@ int laser_game(int DIF_LEVEL)
 	tux_same_counter = 0;
 	ans_num = 0;
 
-	audioMusicPlay(musics[MUS_GAME + int_rand(0,3)], 0);
+	MusicPlay(musics[MUS_GAME + (rand() % NUM_MUSICS)], 0);
 
 	do {
 
@@ -218,7 +172,8 @@ int laser_game(int DIF_LEVEL)
 			} else if (event.type == SDL_KEYDOWN) {
 
 				key = event.key.keysym.sym;
-	      
+				if (key == SDLK_F10) 
+					SwitchScreenMode();	      
 				if (key == SDLK_F11)
 					SDL_SaveBMP( screen, "laser.bmp");
 
@@ -229,13 +184,30 @@ int laser_game(int DIF_LEVEL)
 				if (level_start_wait > 0) 
 					key = SDLK_UNKNOWN;
 				
-				if (((event.key.keysym.unicode & 0xff)>=97) & ((event.key.keysym.unicode & 0xff)<=122)) {
-					ans[ans_num++] = KEYMAP[(event.key.keysym.unicode & 0xff)-32];
-					tux_pressing ++;
-				}else{
-					ans[ans_num++] = KEYMAP[event.key.keysym.unicode & 0xff];
-					tux_pressing ++;
+
+				key_unicode = event.key.keysym.unicode & 0xff;
+
+				DEBUGCODE
+				{
+				  fprintf(stderr, "key_unicode = %d\n", key_unicode);
 				}
+
+				/* For now, tuxtype is case-insensitive for input, */
+                                /* with only uppercase for answers:                */
+                                if (key_unicode >= 97 && key_unicode <= 122)
+                                  key_unicode -= 32;  //convert lowercase to uppercase
+                                if (key_unicode >= 224 && key_unicode <= 255)
+                                  key_unicode -= 32; //same for non-US chars
+
+				LOG ("After checking for lower case:\n");
+				DEBUGCODE
+				{
+				  fprintf(stderr,
+                                   "key_unicode = %d\n", key_unicode);
+				}
+				/* Now update with case-folded value: */
+				ans[ans_num++] = key_unicode;
+
 			}
 		}
       
@@ -244,14 +216,18 @@ int laser_game(int DIF_LEVEL)
 
 		for (;ans_num>0;ans_num--) {
 
-			/*  Pick the lowest comet which has the right answer: */
+			/*  Pick the lowest shootable comet which has the right answer: */
 	
 			lowest_y = 0;
 			lowest = -1;
 	
 			for (i = 0; i < MAX_COMETS; i++)
-				if (comets[i].alive && comets[i].expl == 0 && 
-				    KEYMAP[comets[i].ch] == ans[ans_num-1] && comets[i].y > lowest_y) {
+				if (comets[i].alive
+				 && comets[i].shootable 
+				 && comets[i].expl == 0
+				 && comets[i].ch == ans[ans_num -1 ] 
+				 && comets[i].y > lowest_y)
+				{
 					lowest = i;
 					lowest_y = comets[i].y;
 				}
@@ -264,9 +240,12 @@ int laser_game(int DIF_LEVEL)
 				/* Destroy comet: */
 		  
 				comets[lowest].expl = COMET_EXPL_START;
-	    
-				/* Fire laser: */
+				/* Make next letter in word shootable: */
+				comets[lowest].shootable = 0;
+                                if (comets[lowest].next)
+                                  comets[lowest].next->shootable = 1;
 
+				/* Fire laser: */
 				laser.alive = LASER_START;
 
 				/* this is a hack so drawing to the center of the screen works */
@@ -284,14 +263,14 @@ int laser_game(int DIF_LEVEL)
 					laser.y2 = comets[lowest].y;
 				}
 	    
-				playsound(sounds[SND_LASER]);
+				PlaySound(sounds[SND_LASER]);
 	    
 				/* 50% of the time.. */
 	    
-				if (int_rand(0,2) == 0) {
+				if (0 == (rand() % 2))  {
 
 					/* ... pick an animation to play: */ 
-					if (int_rand(0,2) == 0)
+					if (0 == (rand() % 2))
 						tux_anim = IMG_TUX_YES1;
 					else
 						tux_anim = IMG_TUX_YAY1;
@@ -301,15 +280,15 @@ int laser_game(int DIF_LEVEL)
 
 				/* Increment score: */
 
-				laser_add_score( (DIF_LEVEL+1) * 5 * ((screen->h - comets[lowest].y)/20 ));
+				laser_add_score( (diff_level+1) * 5 * ((screen->h - comets[lowest].y)/20 ));
 
 			} else {
 
 				/* Didn't hit anything! */
 	    
-				playsound(sounds[SND_BUZZ]);
+				PlaySound(sounds[SND_BUZZ]);
 	    
-				if (int_rand(0,2)==0)
+				if (0 == (rand() % 2))
 					tux_img = IMG_TUX_DRAT;
 				else
 					tux_img = IMG_TUX_YIPE;
@@ -333,7 +312,7 @@ int laser_game(int DIF_LEVEL)
 				tux_img = IMG_TUX_SIT;
 	  
 			if (level_start_wait == LEVEL_START_WAIT_START / 4)
-				playsound(sounds[SND_ALARM]);
+				PlaySound(sounds[SND_ALARM]);
 		}
 
       
@@ -341,7 +320,7 @@ int laser_game(int DIF_LEVEL)
 	  
 		if (tux_pressing) {
 			while (tux_img == old_tux_img)
-				tux_img = IMG_TUX_CONSOLE1 + int_rand(0,3);
+				tux_img = IMG_TUX_CONSOLE1 + (rand() % 3);
 
 			playsound(sounds[SND_CLICK]);
 		}
@@ -392,12 +371,12 @@ int laser_game(int DIF_LEVEL)
 		      
 					if (cities[comets[i].city].shields) {
 						cities[comets[i].city].shields = 0;
-						playsound(sounds[SND_SHIELDSDOWN]);
-						laser_add_score(-500 * (DIF_LEVEL+1));
+						PlaySound(sounds[SND_SHIELDSDOWN]);
+						laser_add_score(-500 * (diff_level+1));
 					} else {
 						cities[comets[i].city].expl = CITY_EXPL_START;
-						playsound(sounds[SND_EXPLOSION]);
-						laser_add_score(-1000 * (DIF_LEVEL+1));
+						PlaySound(sounds[SND_EXPLOSION]);
+						laser_add_score(-1000 * (diff_level+1));
 					}
 
 					tux_anim = IMG_TUX_FIST1;
@@ -432,9 +411,9 @@ int laser_game(int DIF_LEVEL)
 
 				/* More comets to add during this wave! */
 		
-				if ((num_comets_alive < 2 || int_rand(0,4) == 0) && distanceMoved > 40) {
+				if ((num_comets_alive < 2 || ((rand() % 4) == 0)) && distanceMoved > 40) {
 					distanceMoved = 0;
-					laser_add_comet(DIF_LEVEL);
+					laser_add_comet(diff_level);
 					num_attackers--;
 				}
 			} else {
@@ -448,7 +427,7 @@ int laser_game(int DIF_LEVEL)
 
 						/* Go on to the next wave: */
 						wave++;
-						laser_reset_level(DIF_LEVEL);
+						laser_reset_level(diff_level);
 
 					} else {
 
@@ -553,7 +532,7 @@ int laser_game(int DIF_LEVEL)
       
 		/* Draw cities: */
       
-		if (frame%2 == 0) next_frame( shield );
+		if (frame%2 == 0) NEXT_FRAME(shield);
 		for (i = 0; i < NUM_CITIES; i++) {
 
 			/* Decide which image to display: */
@@ -637,8 +616,8 @@ int laser_game(int DIF_LEVEL)
       
 		/* Keep playing music: */
       
-		if (sys_sound && !Mix_PlayingMusic())
-			audioMusicPlay(musics[MUS_GAME + int_rand(0,3)], 0);
+		if (settings.sys_sound && !Mix_PlayingMusic())
+			MusicPlay(musics[MUS_GAME + (rand() % NUM_MUSICS)], 0);
       
 		/* Pause (keep frame-rate event) */
       
@@ -655,7 +634,7 @@ int laser_game(int DIF_LEVEL)
 		SDL_FreeSurface(bkgd);
 
 	/* Stop music: */
-	if ((sys_sound) && (Mix_PlayingMusic()))
+	if ((settings.sys_sound) && (Mix_PlayingMusic()))
 		Mix_HaltMusic();
  
 	laser_unload_data();
@@ -664,9 +643,71 @@ int laser_game(int DIF_LEVEL)
 }
 
 
+/*****************************************************/
+/*                                                   */
+/*          Local ("private") functions:             */
+/*                                                   */
+/*****************************************************/
+
+
+
+
+/* --- Load all media --- */
+static void laser_load_data(void)
+{
+	int i;
+
+	/* Create the SDL_Surfaces for all of the characters */
+        /* used in the word list: */
+	font = LoadFont(settings.theme_font_name, COMET_ZAP_FONT_SIZE);
+	RenderLetters(font);
+
+	/* Load images: */
+	for (i = 0; i < NUM_IMAGES; i++) 
+		images[i] = LoadImage(image_filenames[i], IMG_ALPHA);
+	shield = LoadSprite( "cities/shield", IMG_ALPHA );
+
+	if (settings.sys_sound) {
+		for (i = 0; i < NUM_SOUNDS; i++)
+			sounds[i] = LoadSound(sound_filenames[i]);
+
+		for (i = 0; i < NUM_MUSICS; i++)
+			musics[i] = LoadMusic(music_filenames[i]);
+	}
+
+//	PauseLoadMedia();
+}
+
+
+/* --- unload all media --- */
+static void laser_unload_data(void) {
+	int i;
+
+	FreeLetters();
+
+	for (i = 0; i < NUM_IMAGES; i++)
+		SDL_FreeSurface(images[i]);
+
+	if (settings.sys_sound) {
+		for (i = 0; i < NUM_SOUNDS; i++)
+			Mix_FreeChunk(sounds[i]);
+		for (i = 0; i < NUM_MUSICS; i++)
+			Mix_FreeMusic(musics[i]);
+	}
+
+	FreeSprite(shield);
+        shield = NULL;
+
+//	PauseUnloadMedia();
+
+	TTF_CloseFont(font);
+        font = NULL;
+}
+
+
 /* Reset stuff for the next level! */
 
-void laser_reset_level(int DIF_LEVEL)
+static void laser_reset_level(int diff_level)
 {
   unsigned char fname[1024];
   static int last_bkgd = -1;
@@ -678,16 +719,23 @@ void laser_reset_level(int DIF_LEVEL)
     comets[i].alive = 0;
   
   /* Load diffrent random background image: */
+  LOG("Loading background in laser_reset_level()\n");
 
   do {
-    i = int_rand(0,NUM_BKGDS-1);
+    i = rand() % NUM_BKGDS;
+    DOUT(i);
   }
   while (i == last_bkgd);
 
   last_bkgd = i;
 
+  DOUT(i);
+
   sprintf(fname, "backgrounds/%d.jpg", i);
 
+  LOG("Will try to load file:");
+  LOG(fname);
+
   if (bkgd != NULL)
     SDL_FreeSurface(bkgd);
 
@@ -708,106 +756,141 @@ void laser_reset_level(int DIF_LEVEL)
 
   /* Set number of attackers & speed for this wave: */
 
-  switch (DIF_LEVEL) {
+  switch (diff_level) {
     case 0 : speed = 1 + (wave/5); num_attackers=15; break;
     case 1 : speed = 1 + (wave/4); num_attackers=15; break;
     case 2 : speed = 1 + ((wave<<1)/3); num_attackers=(wave<<1); break;
     case 3 : speed = 1 + wave; num_attackers=(wave<<1); break;
+    default: LOG("diff_level not recognized!\n");
   }
 
   distanceMoved = 100; // so that we don't have to wait to start the level
+  LOG("Leaving laser_reset_level()\n");
 }
 
 
 /* Add an comet to the game (if there's room): */
 
-void laser_add_comet(int DIF_LEVEL) {
+static void laser_add_comet(int diff_level) {
 
 	int target, location = 0;
 	static int last = -1;
 	int targeted[NUM_CITIES] = { 0 };
-	int add = int_rand(1,DIF_LEVEL+2);
+	int add = (rand() % (diff_level + 2));
 
-	DEBUGCODE { printf(" Adding %d comets \n", add); }
+	LOG ("Entering laser_add_comet()\n");
+	DEBUGCODE { fprintf(stderr, "Adding %d comets \n", add); }
 
-	if (NUM_CITIES%2 == 0) {
-	while ((add > 0) && (location != MAX_COMETS)) {
-  
-  		/* Look for a free comet slot: */
- 		while ((comets[location].alive == 1) && (location < MAX_COMETS))
-			location++; 
+	if (0 == NUM_CITIES % 2) /* Even number of cities */
+	{
+          LOG("NUM_CITIES is even\n");
+	  while ((add > 0) && (location != MAX_COMETS))
+	  {
+            /* Look for a free comet slot: */
+            while ((comets[location].alive == 1) && (location < MAX_COMETS))
+            {
+              location++; 
+            }
+            if (location < MAX_COMETS)
+            {
+              comets[location].alive = 1;
+              /* Pick a city to attack: */
+              do
+              { 
+                target = (rand() % NUM_CITIES);
+              } while (target == last || targeted[target] == 1);
 
-  		if (location < MAX_COMETS) {
+              last = target;
+              targeted[target] = 1;
 
-			comets[location].alive = 1;
+              /* Set comet to target that city: */
+              comets[location].city = target; 
 
-			/* Pick a city to attack: */
-    
-			do { 
-				target = int_rand( 0, NUM_CITIES );
-    			} while (target == last || targeted[target] == 1);
+              /* Start at the top, above the city in question: */
+              comets[location].x = cities[target].x;
+              comets[location].y = 0;
 
-			last = target;
-			targeted[target] = 1;
+              /* Pick a letter */
+              comets[location].ch = GetLetter();
+              /* single letters always shootable: */
+              comets[location].shootable = 1;
+              comets[location].next = NULL;
 
-			/* Set in to attack that city: */
-      
-			comets[location].city = target; 
+              add--;
+            }
+            DEBUGCODE {if (location == MAX_COMETS) 
+			printf("Location == MAX_COMETS, we have max on screen\n");}
+	  } 
+	}
+	else /* Odd number of cities (is this a hack that means we are using words?) */
+        {
+          LOG("NUM_CITIES is odd\n");
+          wchar_t* word = GetWord();
+          int i = 0;
+          comet_type* prev_comet = NULL;
 
-			/* Start at the top, above the city in question: */
-      
-			comets[location].x = cities[target].x;
-			comets[location].y = 0;
+          DEBUGCODE {fprintf(stderr, "word is: %S\tlength is: %d\n", word, (int)wcslen(word));}
+          do
+          { 
+  	    target = rand() % (NUM_CITIES - wcslen(word) + 1);
+          } while (target == last);
+          last = target;
 
-			/* Pick a letter */
-    
-			comets[location].ch = get_letter();
-			add--;
-		}
-		DEBUGCODE {
-  			if (location == MAX_COMETS) 
-				printf("Location == MAX_COMETS, we have max on screen\n"); 
-		}
-	} }
-	else {
-		unsigned char *word = WORDS_get();
-		int i=0;
-		do { 
-			target = int_rand( 0, NUM_CITIES-strlen(word)+1);
-    		} while (target == last);
-		last = target;
-
-		for (i=0; i<strlen(word); i++) {
+		for (i=0; i < wcslen(word); i++)
+		{
  			while ((comets[location].alive == 1) && (location < MAX_COMETS))
 				location++; 
 
-  			if (location < MAX_COMETS) {
+  			if (location < MAX_COMETS)
+			{
+				/* First comet in word is shootable: */
+				if (0 == i)
+				  comets[location].shootable = 1;
+				else
+				  comets[location].shootable = 0;
 
 				comets[location].alive = 1;
-				comets[location].city = target+i; 
-				comets[location].x = cities[target+i].x;
+				comets[location].city = target + i; 
+				comets[location].x = cities[target + i].x;
 				comets[location].y = 0;
 				comets[location].ch = word[i];
+				comets[location].next = NULL;
+
+				/* Take care of link from previous letter's comet: */
+				if (prev_comet)
+				  prev_comet->next = &comets[location];
+				/* Save pointer for next time through: */
+                                prev_comet = &comets[location];
+
+				DEBUGCODE {fprintf(stderr, "Assigning letter to comet: %C\n", word[i]);}
 			}
 		}
 	}
+	LOG ("Leaving laser_add_comet()\n");
 }
 
 
 /* Draw numbers/symbols over the attacker: */
 
-void laser_draw_let(unsigned char c, int x, int y)
+static void laser_draw_let(wchar_t c, int x, int y)
 {
-	SDL_Rect dst;
-	dst.y = y-35;
-	dst.x = x - (letters[(int)c]->w/2);
-	SDL_BlitSurface(letters[(int)c], NULL, screen, &dst); 
+  /* Draw letter in correct place relative to comet: */
+  const int offset_x = -10; /* Values determined by trial and error: */
+  const int offset_y = -50;
+
+  SDL_Rect dst;
+  SDL_Surface* s;
+  dst.x = x + offset_x;
+  dst.y = y + offset_y;
+  s = GetWhiteGlyph(c);
+  if (s)
+    SDL_BlitSurface(s, NULL, screen, &dst); 
 }
 
 
 /* Draw status numbers: */
 
-void laser_draw_numbers(unsigned char * str, int x)
+static void laser_draw_numbers(const unsigned char* str, int x)
 {
   int i, cur_x, c;
   SDL_Rect src, dest;
@@ -856,7 +939,7 @@ void laser_draw_numbers(unsigned char * str, int x)
 
 /* Draw a line: */
 
-void laser_draw_line(int x1, int y1, int x2, int y2, int red, int grn, int blu)
+static void laser_draw_line(int x1, int y1, int x2, int y2, int red, int grn, int blu)
 {
   int dx, dy, tmp;
   float m, b;
@@ -909,7 +992,7 @@ void laser_draw_line(int x1, int y1, int x2, int y2, int red, int grn, int blu)
 
 /* Draw a single pixel into the surface: */
 
-void laser_putpixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
+static void laser_putpixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
 {
 #ifdef PUTPIXEL_RAW
   int bpp;
@@ -974,7 +1057,7 @@ void laser_putpixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
 
 /* Draw image at lower center of screen: */
 
-void laser_draw_console_image(int i)
+static void laser_draw_console_image(int i)
 {
   SDL_Rect dest;
 
@@ -989,7 +1072,7 @@ void laser_draw_console_image(int i)
 
 /* Increment score: */
 
-void laser_add_score(int inc)
+static void laser_add_score(int inc)
 {
   score += inc;
   if (score < 0) score = 0;
